/* Copyright (c) Determina Corp. 2007.  All rights reserved.
 * 
 * This software and documentation constitute an unpublished work and contain
 * valuable trade secrets and proprietary information belonging to Determina
 * Corp.  None of the foregoing material may be copied, duplicated or
 * disclosed without the express written permission of Determina Corp.
 * DETERMINA CORP. EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING THIS
 * SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF MERCHANTABILITY
 * AND/OR FITNESS FOR ANY PARTICULAR PURPOSE, AND WARRANTIES OF PERFORMANCE,
 * AND ANY WARRANTY THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE
 * OF TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO THE USE
 * OF THE SOFTWARE OR DOCUMENTATION. Under no circumstances shall Determina
 * Corp. be liable for incidental, special, indirect, direct or consequential
 * damages or loss of profits, interruption of business, or related expenses
 * which may arise from use of software or documentation, including but not
 * limited to those resulting from defects in software and/or documentation,
 * or loss or inaccuracy of data of any kind.
 *
 * Determina Fix for CVE-2007-0038
 *
 */

#include <windows.h>

#include "patch.h"


//
// Modules that we watch to patch
//

struct module modules[] = {

    // Windows 2000 MS05-002

    {
        "USER32.dll",
        { 0x2B31D, 0x45, 1978929905 },
        0x2b33d
    },

    // Windows 2000 MS05-018

    {
        "USER32.dll",
        { 0x2B2CD, 0x45, 1978929905 },
        0x2b2ed
    },

    // Windows 2000 SP4 UR1

    {
        "USER32.dll",
        { 0x463EA, 0x45, 4060453513 },
        0x4640a
    },

    // Windows XP SP1 MS05-002 qfe

    {
        "USER32.dll",
        { 0x36AB8, 0x45, 2918002184 },
        0x36ad8
    },

    // Windows XP SP1 MS05-018 qfe, MS05-053 qfe

    {
        "USER32.dll",
        { 0x34BBA, 0x45, 3406939800 },
        0x34bda
    },

    // Windows XP SP2

    {
        "USER32.dll",
        { 0x43FFC, 0x45, 1814692103 },
        0x4401c
    },

    // Windows XP SP2 MS05-018 gdr

    {
        "USER32.dll",
        { 0x43FBD, 0x45, 379109036 },
        0x43fdd
    },

    // Windows XP SP2 MS05-018 qfe

    {
        "USER32.dll",
        { 0x43FC5, 0x45, 379109036 },
        0x43fe5
    },

    // Windows 2003 SP0 MS05-002 gdr

    {
        "USER32.dll",
        { 0x469B5, 0x45, 255403783 },
        0x469d5
    },

    // Windows 2003 SP0 MS05-002 qfe

    {
        "USER32.dll",
        { 0x46B65, 0x45, 255403783 },
        0x46b85
    },

    // Windows 2003 SP1

    {
        "USER32.dll",
        { 0x4259A, 0x45, 132454611 },
        0x425ba
    },

    // Windows 2003 SP2

    {
        "USER32.dll",
        { 0x47BBC, 0x45, 3935119632 },
        0x47bdc
    }
};

int num_modules = sizeof(modules)/sizeof(struct module);

// Patch status

enum patch_status {
    NOT_PATCHED,
    PATCHED
};

extern "C" {
    __declspec(dllexport) enum patch_status patch_status = NOT_PATCHED;
}


// Address of the ReadChunk function in USER32.DLL
void* ReadChunk = NULL;

//
// ReadChunk hook
//
// This hook intercepts calls to the ReadChunk function from LoadAniIcon.
//
// The pointer to the chunk structure will be at POI(ESP+8). We will check the
// chunk size field at POI(POI(ESP+8)+4). If it is not 36, we will skip the
// call to ReadChunk and return 0.

__declspec(naked) void ReadChunk_hook()
{
    __asm {
        push eax                // save eax

        mov eax, [esp+0x8]      // pointer to chunk structure
        mov eax, [eax+0x4]      // pointer to chunk size
        mov eax, [eax]          // chunk size
        cmp eax, 0x24           // if (chunk size == 0x24)
        jz read_chunk           //     goto read_chunk

        pop eax
        xor eax, eax            // return 0
        ret 0xC

read_chunk:
        pop eax                 // restore eax
        jmp [ReadChunk]         // jump to the DSA_SetItem function
    }
}

// We're going to patch the call to ReadChunk in LoadAniIcon in USER32.DLL
//
// 77D8401C   E8 E4 FA FF FF                  call    _ReadChunk@12
//
// Our patch will modify the operand of the call instruction and replace it
// with the address of our ReadChunk_hook function.
//
// The hook function will check the chunk size, and if it's not 36 it will
// return 0 without calling the real ReadChunk function.

// Opcode of the instruction we're patching
#define CALL_REL32 "\xE8"           // call rel32

//
// Patch the module
//

void patch_module(struct module* module, char* image_base)
{
    // Get the absolute addresses of the patch point

    char* patch_point = image_base + module->patch_point;

    // Make sure that the patch point starts with a call ds:address instruction

    if (memcmp(patch_point, CALL_REL32, sizeof(CALL_REL32)-1) != 0)
        return;
    
    // We want to patch the address in the call instruction

    patch_point += 1;
    
    // Get the address of the ReadChunk function

    ReadChunk = (patch_point + 4) + *(unsigned int*)patch_point;
    
    // Change the memory protection of the patch point to enable writing
    
    DWORD old_protection, dummy;

    if (VirtualProtect(patch_point, sizeof(void*), PAGE_EXECUTE_READWRITE, &old_protection) == 0) {
        return;
    }

    // Modify the call instruction to call ReadChunk_hook
    *(unsigned int*)patch_point = (unsigned int)&ReadChunk_hook - ((unsigned int)patch_point + 4);

    // Restore the memory protection of the patch point

    VirtualProtect(patch_point, sizeof(void*), old_protection, &dummy);

    patch_status = PATCHED;
}
