/* Copyright (c) Determina Corp. 2007.  All rights reserved.
 * 
 * This software and documentation constitute an unpublished work and contain
 * valuable trade secrets and proprietary information belonging to Determina
 * Corp.  None of the foregoing material may be copied, duplicated or
 * disclosed without the express written permission of Determina Corp.
 * DETERMINA CORP. EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING THIS
 * SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF MERCHANTABILITY
 * AND/OR FITNESS FOR ANY PARTICULAR PURPOSE, AND WARRANTIES OF PERFORMANCE,
 * AND ANY WARRANTY THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE
 * OF TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO THE USE
 * OF THE SOFTWARE OR DOCUMENTATION. Under no circumstances shall Determina
 * Corp. be liable for incidental, special, indirect, direct or consequential
 * damages or loss of profits, interruption of business, or related expenses
 * which may arise from use of software or documentation, including but not
 * limited to those resulting from defects in software and/or documentation,
 * or loss or inaccuracy of data of any kind.
 */


#include <windows.h>

// Product name

#define PATCH_NAME "Determina Fix for CVE-2007-0038"

// Patch and target DLL names

#define PATCH_DLL  "CVE-2007-0038.DLL"
#define TARGET_DLL "USER32.DLL"

enum patch_status {
    NOT_PATCHED,
    PATCHED
};

//
// Main function
//

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
    UNREFERENCED_PARAMETER(hInstance);
    UNREFERENCED_PARAMETER(hPrevInstance);
    UNREFERENCED_PARAMETER(lpCmdLine);
    UNREFERENCED_PARAMETER(nCmdShow);

    // Make sure the patch DLL is loaded

    HMODULE patch_dll = GetModuleHandle(PATCH_DLL);

    if (patch_dll == NULL) {
        MessageBox(NULL, PATCH_DLL " is not loaded. Please make sure that it is listed in the AppInit_DLLs registry key.", PATCH_NAME,
                   MB_OK | MB_ICONERROR);
        return 0;
    }

    enum patch_status* patch_status = (enum patch_status*) GetProcAddress(patch_dll, "patch_status");
    if (  patch_status == NULL ||
        (*patch_status != PATCHED && *patch_status != NOT_PATCHED))
    {
        MessageBox(NULL, "Unable to read protection status. Please contact Determina.", PATCH_NAME, MB_OK | MB_ICONERROR);
        return 0;
    }

    // Load the target DLL

    if (LoadLibrary(TARGET_DLL) == NULL) {
        MessageBox(NULL, "Unable to load " TARGET_DLL, PATCH_NAME,
                   MB_OK | MB_ICONERROR);
        return 0;
    }

    if (*patch_status == PATCHED) {
        MessageBox(NULL, "The " PATCH_NAME " is active.", PATCH_NAME, MB_OK);
    }
    else {
        MessageBox(NULL, "The " PATCH_NAME " could not be applied.\n\nIf you are sure that you are running a vulnerable version of " TARGET_DLL ", please contact Determina.", PATCH_NAME, MB_OK | MB_ICONERROR);
    }

    return 0;
}
